<?php
/*
Plugin Name: Plugin Information
Plugin URI: https://blog.tremlas.com/wordpress-plugins/plugin-information-plugin-details/
Description: Adds a shortcode that inserts the equivalent of the plugin information popup available to admins
Version: 3.126.0
Author: David Vines
Author URI: http://blog.tremlas.com
License:  GPLv2 or later
*/
/* © Copyright 2021 David Vines

Plugin Information is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
any later version.

Plugin Information is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with {Plugin Name}. If not, see {URI to Plugin License}.*/

// Debugging function in case we need it
if ( !function_exists('djv_write_log') ) {
	function djv_write_log ( $log )  {
		if ( true === WP_DEBUG ) {
			if ( is_array( $log ) || is_object( $log ) ) {
				error_log( print_r( $log, true ) );
			} else {
				error_log( $log );
			}
		}
	}
}

/**
 * Shortcode to add the version of the plugin
 */
function djv_pluginInformation_pluginInformationVersion_shortcode() {
	return 'Plugin Information Version 3.126.0';
}
add_shortcode( 'pluginInformationVersion', 'djv_pluginInformation_pluginInformationVersion_shortcode' );

/**
 * Inserts the equivalent of the plugin information pages for a plugin. The only supported attribute is 'slug'
 */

// Now for a shortcode that can be used to get links and/or tables for the characters
function djv_pluginInformation_shortcode( $attr, $content = null ) {
	global $wp;
	global $wpdb;

	if ( isset($attr) && isset( $attr['slug'] ) ) {
		$slug = $attr['slug'];
	} else {
		return "No slug attribute provided on the pluginInformation shortcode";
	}

	$attach = ( isset($attr) && isset( $attr['attachment'] ) && filter_var( $attr['attachment'], FILTER_VALIDATE_BOOLEAN ) );

	// Remove unnecessary paragraph wrapping
	if (! empty( $content ) ) {
		$patterns = array('/^<\/p>/i', '/<p>$/i');
		$content = preg_replace($patterns, '', $content);
	}

	$api = djv_pluginInformation_get_remote_update_info( $slug );

	if ( is_wp_error( $api ) ) {
		return 'Error retrieving the information for the plugin with slug "' . $slug . '"';
	}

	if ( empty($api->sections) ) {
		return "<p>No information successfully retrieved using <q>" . $slug . "</q></p><p>We got </p><pre>" . print_r( $api, true ) . "</pre>";
	}

	$plugins_allowedtags = array(
			'a'          => array(
					'href'   => array(),
					'title'  => array(),
					'target' => array(),
			),
			'abbr'       => array( 'title' => array() ),
			'acronym'    => array( 'title' => array() ),
			'code'       => array(),
			'pre'        => array(),
			'em'         => array(),
			'strong'     => array(),
			'div'        => array( 'class' => array() ),
			'span'       => array( 'class' => array() ),
			'p'          => array(),
			'br'         => array(),
			'ul'         => array(),
			'ol'         => array(),
			'li'         => array(),
			'h1'         => array(),
			'h2'         => array(),
			'h3'         => array(),
			'h4'         => array(),
			'h5'         => array(),
			'h6'         => array(),
			'img'        => array(
					'src'   => array(),
					'class' => array(),
					'alt'   => array(),
			),
			'blockquote' => array( 'cite' => true ),
	);

	// Sanitize HTML.
	foreach ( array( 'version', 'author', 'requires', 'tested', 'homepage', 'downloaded', 'slug' ) as $key ) {
		if ( isset( $api->$key ) ) {
			$api->$key = wp_kses( $api->$key, $plugins_allowedtags );
		}
	}

	$text  = '<table class="plugin-info-table"><tbody><tr><td  class="plugin-info-table-description">' . PHP_EOL;
	// Sansitize the description HTML
	if ( ! empty($api->sections) ) {
		$text .= wp_kses( $api->sections[ 'description' ], $plugins_allowedtags ) . PHP_EOL;
	}
	$text .= '</td><td class="plugin-info-table-fyi">' . PHP_EOL;

	$text .= '<ul>' . PHP_EOL;
	if ( ! empty( $api->version ) ) {
		$text .= '<li><strong>Version:</strong> ';
		if ( ! empty( $api->download_link ) ) {
			if ( $attach ) {
				$id = $wpdb->get_var( "SELECT ID FROM $wpdb->posts WHERE guid LIKE '%" . $api->download_link ."'" );
				$text .= '<a href="' . get_attachment_link( $id ) . '">' . $api->version . '</a>' . PHP_EOL;
			} else {
				$text .= '<a href="' . $api->download_link . '">' . $api->version . '</a>' . PHP_EOL;
			}
		} else {
			$text .= $api->version;
		}
		$text .= '</li>' . PHP_EOL;
	}
	if ( ! empty( $api->author ) ) {
		$text .= '<li><strong>Author:</strong> ' . links_add_target( $api->author, '_blank' ) . '</li>' . PHP_EOL;
	}
	if ( ! empty( $api->last_updated ) ) {
		$text .= '<li><strong>Last Updated:</strong> ' . human_time_diff( strtotime( $api->last_updated ) ) . ' ago</li>' . PHP_EOL;
	}
	if ( ! empty( $api->requires ) ) {
		$text .= '<li><strong>Requires WordPress Version:</strong> ' . $api->requires . ' or higher</li>' . PHP_EOL;
	}
	if ( ! empty( $api->tested ) ) {
		$text .= '<li><strong>Compatible up to:</strong> ' . $api->tested . '</li>' . PHP_EOL;
	}
	if ( ! empty( $api->requires_php ) ) {
		$text .= '<li><strong>Requires PHP Version:</strong> ' . $api->requires_php . ' or higher</li>' . PHP_EOL;
	}
	if ( ! empty( $api->homepage ) ) {
		$mustMatch = array_flip(['host', 'port', 'path']);
		$defaults  = ['path' => '/']; // if not present, assume these
		$url1ai = array_intersect_key( parse_url( home_url( $wp->request ) ), $mustMatch ) + $defaults;
		$url2ai = array_intersect_key( parse_url( $api->homepage ), $mustMatch ) + $defaults;
		$url1ai['path'] = rtrim( $url1ai['path'], '/');
		$url2ai['path'] = rtrim( $url2ai['path'], '/');

		if ( $url1ai !== $url2ai ) {
			$text .= '<li><strong>Plugin homepage:</strong> <a target="_blank" href="' . esc_url( $api->homepage ) . '">click here &#187;</a></li>' . PHP_EOL;
		}
	}
	if ( ! empty( $content ) ) {
		$text .= '<li>' . $content  . '</li>' . PHP_EOL;
	}
	$text .= '</ul></td></tr></table>' . PHP_EOL;

	return $text;
}

add_shortcode( 'pluginInformation', 'djv_pluginInformation_shortcode' );

function djv_pluginInformation_changeLog_shortcode( $attr, $content = null ) {
	global $wpdb;
	if ( isset($attr) && isset( $attr['slug'] ) ) {
		$slug = $attr['slug'];
	} else {
		return "No slug attribute provided on the pluginInformation shortcode";
	}

	$attach = ( isset($attr) && isset( $attr['attachment'] ) && filter_var( $attr['attachment'], FILTER_VALIDATE_BOOLEAN ) );

	$api = djv_pluginInformation_get_remote_update_info( $slug );

	if ( is_wp_error( $api ) ) {
		return 'Error retrieving the information for the plugin with slug "' . $slug . '"';
	}

	if ( empty($api->sections) ) {
		return "<p>No information successfully retrieved using <q>" . $slug . "</q></p><p>We got </p><pre>" . print_r( $api, true ) . "</pre>";
	}

	$plugins_allowedtags = array(
			'a'          => array(
					'href'   => array(),
					'title'  => array(),
					'target' => array(),
			),
			'abbr'       => array( 'title' => array() ),
			'acronym'    => array( 'title' => array() ),
			'code'       => array(),
			'pre'        => array(),
			'em'         => array(),
			'strong'     => array(),
			'div'        => array( 'class' => array() ),
			'span'       => array( 'class' => array() ),
			'p'          => array(),
			'br'         => array(),
			'ul'         => array(),
			'ol'         => array(),
			'li'         => array(),
			'h1'         => array(),
			'h2'         => array(),
			'h3'         => array(),
			'h4'         => array(),
			'h5'         => array(),
			'h6'         => array(),
			'img'        => array(
					'src'   => array(),
					'class' => array(),
					'alt'   => array(),
			),
			'blockquote' => array( 'cite' => true ),
	);

	if ( ! empty($api->sections) ) {
		$text = wp_kses( $api->sections[ 'changelog' ], $plugins_allowedtags ) . PHP_EOL;
	}

	if ( $attach ) {
		$dom = new DOMDocument();
		$dom->loadHTML( '<div>' . $text . '</div>' );
		$links = $dom->getElementsByTagName( 'a' );
		foreach($links as $link) {
			$url = $link->getAttribute( 'href' );
			$id = $wpdb->get_var( "SELECT ID FROM $wpdb->posts WHERE guid LIKE '%" . $url ."'" );
			$link->setAttribute( 'href', get_attachment_link( $id ) );
		}
		$text = substr($dom->saveXML($dom->getElementsByTagName('div')->item(0)), 5, -6);
	}

	return $text;
}
add_shortcode( 'pluginInformationChangeLog', 'djv_pluginInformation_changeLog_shortcode' );


/* --------------------- Functions for updating plugins ------------------------------ */
function djv_pluginInformation_get_remote_update_from( $urlbase, $slug ) {
 $url = $urlbase . '/wp-content/uploads/jsons/' . $slug . '.json';
 $remote =  wp_remote_get( $url, array( 'timeout' => 10, 'headers' => array( 'Accept' => 'application/json' ) ) );
 if ( !is_wp_error( $remote ) && isset( $remote['response']['code'] ) && 200 == $remote['response']['code'] && ! empty( $remote['body'] ) ) {
  $remote['urlbase'] = $urlbase;
 }
 return $remote;
}
function djv_pluginInformation_get_remote_update_info( $slug ) {
 $remote = get_transient( 'djv_update_' . $slug );
 if (false == $remote || is_wp_error($remote) || !isset( $remote['response']['code'] ) || 200 !== $remote['response']['code'] || empty( $remote['body'] ) ) {
  if ( 'cubi' == gethostname() ) {
   $remote = djv_pluginInformation_get_remote_update_from( 'http://cubi.internal', $slug );
   set_transient( 'djv_update_' . $slug, $remote, 10 ); // Save for only 10 seconds here
  } else {
   $remote = djv_pluginInformation_get_remote_update_from( 'https://blog.tremlas.com', $slug );
   set_transient( 'djv_update_' . $slug, $remote, 43200 ); // Save for 12 hours
  }
 }
 if ( !is_wp_error( $remote ) && isset( $remote['response']['code'] ) && 200 == $remote['response']['code'] && ! empty( $remote['body'] ) ) {
  $body = json_decode( $remote['body'] );
  $info = new StdClass();
  $info->name = 'Plugin Information';
  $info->slug = $slug;
  $info->version = $body->version;
  $info->requires = $body->requires;
  $info->author = '<a href="' . $body->author_profile . '">' . $body->author . '</a>';
  $info->download_link = $remote['urlbase'] . '/' . $body->download_url;
  $info->trunk = $remote['urlbase'] . '/' . $body->download_url;
  $info->last_updated = $body->last_updated;
  $info->sections = array(
    'description'  => implode( "\n", $body->sections->description ),
    'installation' => implode( "\n", $body->sections->installation ),
    'changelog'    => implode( "\n", $body->sections->changelog ),
  );
  $info->homepage = $remote['urlbase'] . '/' . $body->homepage;
  return $info;
 }
 return false;
}

function djv_pluginInformation_check_for_update( $name, $slug ) {
 $info = djv_pluginInformation_get_remote_update_info( $slug );
 if ( false == $info ) {
  return false;
 }
 if ( version_compare( '3.126.0', $info->version, '<' ) && version_compare( $info->requires, get_bloginfo( 'version' ), '<' ) ) {
  $update = (object) array(
    'id'            => $name,
    'slug'          => $slug,
    'plugin'        => $name,
    'new_version'   => $info->version,
    'url'           => $info->homepage,
    'package'       => $info->download_link,
    'icons'         => array(),
    'banners'       => array(),
    'banners_rtl'   => array(),
    'tested'        => '',
    'requires_php'  => '',
    'compatability' => new StdClass()
  );
  return $update;
 } else {
  return false;
 }
}

function djv_pluginInformation_pre_set_site_transient_update_plugins( $transient ) {
 // Check for update
 $pluginphp = 'pluginInformation/pluginInformation.php';
 $pluginslug = 'pluginInformation';
 $update = djv_pluginInformation_check_for_update( $pluginphp, $pluginslug );
 if ( $update ) {
  $transient->response[$pluginphp] = $update;
 } else {
  // No update
  $item = (object) array(
    'id'            => $pluginphp,
    'slug'          => $pluginslug,
    'plugin'        => $pluginphp,
    'new_version'   => '3.126.0',
    'url'           => '',
    'package'       => '',
    'icons'         => array(),
    'banners'       => array(),
    'banners_rtl'   => array(),
    'tested'        => '',
    'requires_php'  => '',
    'compatability' => new StdClass()
  );
  $transient->no_update[$pluginphp] = $item;
 }
 return $transient;
}
add_filter( 'pre_set_site_transient_update_plugins', 'djv_pluginInformation_pre_set_site_transient_update_plugins' );

function djv_pluginInformation_plugin_info( $result, $action, $args ) {
 $slug = 'pluginInformation';
 if ( 'plugin_information' !== $action ) {
  return $result;
 }
 if ( $slug !== $args->slug) {
  return $result;
 }
 return djv_pluginInformation_get_remote_update_info( $slug );
}
add_filter( 'plugins_api', 'djv_pluginInformation_plugin_info', 10, 3 );
?>